require(["esri/map", "esri/layers/TileInfo", "esri/layers/osm", "dojo/domReady!"], function(Map, TileInfo) {
    resize();
    window.onresize = resize;
    //二维地图的显示比率与三维地图摄像机距离的商作为系数，这个为线性关系，根据实际情况计算即可
    var COEFFICIENT = 0.2087304735282085;
    var levelRange = []; //二维地图各等级下对应的三维地图摄像机的距离
    var preRange = 0;
    //确定当前谁是主要地图，谁是跟随地图
    var mode = "none";
    $('#stampMapDiv').mouseenter(function() {
        mode = "3D";
    });
    $('#stampMapDiv').mouseout(function() {
        mode = "none";
    });
    $('#mapDiv').mouseenter(function(event) {
        mode = "2D";
    });
    $('#mapDiv').mouseout(function(event) {
        mode = "none";
    });
    //加载三维球体
    $('#stampMapDiv').empty().html('<object id="seearth" ' +
        'classid="clsid:EA3EA17C-5724-4104-94D8-4EECBD352964" ' +
        'data="data:application/x-oleobject;base64,Xy0TLBTXH0q8GKFyFzl3vgAIAADYEwAA2BMAAA==" ' +
        'width="100%" height="100%"></object>');
    //当object对象创建完毕，会触发下面绑定的事件
    seearth.Event.OnCreateEarth = function() {
        seearth.Event.OnCreateEarth = function() {};
        //开始加载数据
        seearth.Event.OnDocumentChanged = function(type, guid) {
            //因为这个一般只用执行一次，所以当绑定的事件触发后清除绑定
            seearth.Event.OnDocumentChanged = function() {};
            preRange = levelRange[12];
            //修改当前视图的范围已经摄像机视角等，这里是为了演示方便写死进去的，可根据自己数据进行修改，参照api文档
            seearth.GlobeObserver.GotoLookat(117.191039, 34.252835, 32.19071602597978,
                0, 90, 0, preRange);
        };
        //三维球数据主机地址
        seearth.Load('http://192.168.1.127', 0);
    };

    //加载二维地图
    //使用openstreet地图
    var map, osmLayer;
    map = new Map("mapDiv", {
        center: [117.1805, 34.2530],
        zoom: 12,
        logo: false
    });
    osmLayer = new esri.layers.OpenStreetMapLayer();
    map.addLayer(osmLayer);

    //收集二维地图各等级下对应的三维地图的摄像机距离
    var lods = map.__tileInfo.lods;
    for (var i = 0; i < lods.length; i++) {
        levelRange.push(lods[i].scale * COEFFICIENT);
    }

    //二维地图视图范围变化
    var mapExtentChange = map.on("extent-change", changeHandler);

    function changeHandler(evt) {
        var extent = evt.extent,
            zoomed = evt.levelChange;
        if (mode == "2D") {
            var latitude = map.extent.getCenter().getLatitude();
            var longitude = map.extent.getCenter().getLongitude();
            var altitude = seearth.GlobeObserver.GetHeight(longitude, latitude);
            var heading = seearth.GlobeObserver.Pose.Heading;
            var tilt = seearth.GlobeObserver.Pose.Tilt;
            var roll = seearth.GlobeObserver.Pose.Roll;
            var range = seearth.GlobeObserver.Pose.Range;
            //计算摄像机距离
            range = evt.target.getScale() * COEFFICIENT;
            seearth.GlobeObserver.GotoLookat(longitude, latitude, altitude,
                heading, tilt, roll, range);
        }
    }

    //三维地图视图范围发生变化
    seearth.Event.OnObserverChanged = function() {
        if (mode == "3D") {
            var earthPose = seearth.GlobeObserver.TargetPose;
            var latitude = earthPose.Latitude;
            var longitude = earthPose.Longitude;
            var range = seearth.GlobeObserver.Pose.Range;
            //根据当前三维球摄像机距离计算出二维地图的显示比率
            for (var i = 0; i < levelRange.length - 1; i++) {
                if (range >= levelRange[0]) {
                    range = levelRange[0];
                }
                if (range <= levelRange[levelRange.length - 1]) {
                    range = levelRange[levelRange.length - 1];
                }
                var range1 = levelRange[i];
                var range2 = levelRange[i + 1];
                var needBreak = false;
                //缩放时求出距离最近的缩放距离，因为瓦片地图是分级的不是无极缩放的
                if (range <= range1 && range >= range2) {
                    if (range1 - range <= range - range2) {
                        range = range1;
                        map.centerAndZoom([longitude, latitude], i);
                    } else {
                        range = range2;
                        map.centerAndZoom([longitude, latitude], i + 1);
                    }
                    break;
                }
            }
        }
    }
});

//当浏览器窗口尺寸发生变化后，调整布局
function resize() {
    //顶部栏目
    var topHeight = 20;
    var margin = 6;
    var border = 2;
    var height = $("body").height();
    var mapHeight = height - topHeight - margin;
    var width = $("body").width() - 3 * margin - 4 * border;
    $("#mapDiv,#stampMapDiv").width(width / 2).height(mapHeight).css("top", topHeight + "px");
    $("#stampMapDiv").css("left", margin + "px");
    $("#mapDiv").css("left", width * 0.5 + 2 * margin - 4 * border + "px");
}
